package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
 
/**
 * EJB Gen Delegate
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: EJB Delegate to archive EJB Beans along with deployment descriptor
 * 
 * <p>The EJB Delegate generates archive file of an EJB project by including its container specific deployment descriptors
 * along with the Bean classes. This archiving is achieved by using ejbjar ant task. 
 * The generated archive can then be deployed as an EJB module in an EJB Container</p>
 */
/* 
 * @ant-target doEjbGen
 */
class EJBGenDelegate extends AbstractPluginTargetDelegate {

	/*
	 * Base directory under which to scan for EJB deployment descriptors. If not provided, then  the deployment descriptors must be located in the directory specified by srcdir attribute 
	 * @ant-property-name ejb.descriptor.dir
	 */
	protected String descriptorDir
	/*
	 * Version of CMP (either 1.0 or 2.0). A CMP 2.0 implementation exists only for JBoss 
	 * @ant-property-name ejb.cmp.version
	 */
	protected String cmpVersion
	/*
	 * Base directory in to which generated jar files are deposited.
	 * @ant-property-name ejb.destination.dir
	 */
	protected String destinationDir
	/*
	 * Controls the inclusiong of additional classes and interfaces to the archive.
	 * @ant-property-name ejb.dependency
	 */
	protected String dependency
	/*
	 * Base name that is used for the generated jar files. This attribute is used when naming attribute has the value basejarname
	 * @ant-property-name ejb.base.jar.name
	 */
	protected String baseJarName
	/*
	 * String value appended to the base name of the deployment descriptor to create the filename of the generic EJB jar file.
	 * @ant-property-name generic.jar.suffix
	 */
	protected String genericJarSuffix
	/*
	 * Maps to the base directory containing bean classes.Included are the home- remote- pk- and implementation- classes and all classes, that these depend on. Note that this can be the same as the descriptordir if all files are in the same directory tree.
	 * @ant-property-name ejb.src.dir
	 */
	protected String srcDir
	/*
	 * This attribute takes 4 possible values. descriptor attribute value is default naming scheme. Name of the generated Jar file is derived from the deployment descriptor
	 * ejb-name attribute value uses ejb-name element from the deployment descriptor. directory attribute value will derive the name of the generated bean jar from the directory containing 
	 * the deployment descriptor. basejarname attribute value indicates that the name of bean would be explicitly provided
	 * @ant-property-name ejb.naming
	 */
	protected String naming
	/*
	 * String value used to substring out of a string from the name of each deployment descriptor. This is not to be used when using the attribute, basejarname.
	 * @ant-property-name ejb.base.name.terminator
	 */
	protected String baseNameTerminator
	/*
	 * Set this attribute to true if you want all generated jars to be placed in the root of the destdir, rather than according to the location of the deployment descriptor within the descriptor dir hierarchy.
	 * @ant-property-name ejb.flat.dest.dir
	 */
	protected String flatDestDir
	/*
	 * Directory to include additional files using support nested element of ejbjar task
	 * @ant-property-name ejb.support.dir
	 */
	protected String supportDir
	/*
	 * Include pattern in the inclusion directory
	 * @ant-property-name ejb.support.includes
	 */
	protected String supportIncludes
	/*
	 * Exclude pattern in the inclusion directory
	 * @ant-property-name ejb.support.excludes
	 */
	protected String supportExcludes

	/**
	 * Setter method for member variable descriptorDir
	 * @param descriptorDir	
	 */	
	void setDescriptorDir( String descriptorDir ){
		this.descriptorDir = descriptorDir
	}
	/**
	 * Setter method for member variable cmpVersion
	 * @param cmpVersion	
	 */	
	void setCmpVersion( String cmpVersion ){
		this.cmpVersion = cmpVersion
	}
	/**
	 * Setter method for member variable destinationDir
	 * @param destinationDir	
	 */	
	void setDestinationDir( String destinationDir ){
		this.destinationDir = destinationDir
	}
	/**
	 * Setter method for member variable dependency
	 * @param dependency	
	 */	
	void setDependency( String dependency ){
		this.dependency = dependency
	}
	/**
	 * Setter method for member variable baseJarName
	 * @param baseJarName	
	 */	
	void setBaseJarName( String baseJarName ){
		this.baseJarName = baseJarName
	}
	/**
	 * Setter method for member variable genericJarSuffix
	 * @param genericJarSuffix	
	 */	
	void setGenericJarSuffix( String genericJarSuffix ){
		this.genericJarSuffix = genericJarSuffix
	}
	/**
	 * Setter method for member variable srcDir
	 * @param srcDir	
	 */	
	void setSrcDir( String srcDir ){
		this.srcDir = srcDir
	}
	/**
	 * Setter method for member variable naming
	 * @param naming	
	 */	
	void setNaming( String naming ){
		this.naming = naming
	}
	/**
	 * Setter method for member variable baseNameTerminator
	 * @param baseNameTerminator	
	 */	
	void setBaseNameTerminator( String baseNameTerminator ){
		this.baseNameTerminator = baseNameTerminator
	}
	/**
	 * Setter method for member variable flatDestDir
	 * @param flatDestDir	
	 */	
	void setFlatDestDir( String flatDestDir ){
		this.flatDestDir = flatDestDir
	}
	/**
	 * Setter method for member variable supportDir
	 * @param supportDir	
	 */	
	void setSupportDir( String supportDir ){
		this.supportDir = supportDir
	}
	/**
	 * Setter method for member variable supportIncludes
	 * @param supportIncludes	
	 */	
	void setSupportIncludes( String supportIncludes ){
		this.supportIncludes = supportIncludes
	}
	/**
	 * Setter method for member variable supportExcludes
	 * @param supportExcludes	
	 */	
	void setSupportExcludes( String supportExcludes ){
		this.supportExcludes = supportExcludes
	}

	 /**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 EJBGenDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert descriptorDir != null, "Error due to Null value of ejb.descriptor.dir"
		assert cmpVersion != null, "Error due to Null value of ejb.cmp.version"
		assert destinationDir != null, "Error due to Null value of ejb.destination.dir."
		assert dependency != null, "Error due to Null value of ejb.dependency."
		assert baseJarName != null, "Error due to Null value of ejb.base.jar.name."
		assert genericJarSuffix != null, "Error due to Null value of ejb.generic.jar.suffix."
		assert srcDir != null, "Error due to Null value of ejb.src.dir."
		assert naming != null, "Error due to Null value of ejb.naming."
		assert baseNameTerminator != null, "Error due to Null value of ejb.base.name.terminator."
		assert flatDestDir != null, "Error due to Null value of ejb.flat.dest.dir."
		assert supportDir != null, "Error due to Null value of ejb.support.dir."
		assert supportIncludes != null, "Error due to Null value of ejb.support.includes."
		assert supportExcludes != null, "Error due to Null value of ejb.support.excludes."
		def destDir = new File(destinationDir)
		assert (destDir.exists() && destDir.isDirectory()), "ejb.destination.dir should be a valid directory"
	}	
	
	/**
	 * Concrete implementation of the AbstractDelegate.execute() method
	 */
	void doExecute(){
		/* Append destination directory and file name for destination location*/
		def ejbDest = destinationDir + "/" + baseJarName
		
		/* Populate attributes for the ejbjar task */
		def ejbOptions = [ "descriptordir":descriptorDir,
							"srcdir":srcDir,
							"destdir":destinationDir,
							"dependency":dependency,
							"genericjarsuffix":genericJarSuffix,
							"flatdestdir":flatDestDir,
							"cmpversion":cmpVersion]
		
		/* Use the correct attributes based on mutually exclusive attributes */
		if(naming.equals("basejarname")) {
			ejbOptions.put( "basejarname", baseJarName )
		} else {
			ejbOptions.put( "basenameterminator", baseNameTerminator )
		}
		
		/* Execute ejbjar ant task with the loaded attributes*/
		ant.ejbjar(	ejbOptions ) {
			ant.support(dir:supportDir) {
				ant.include(name:supportIncludes)
				ant.exclude(name:supportExcludes)
			}
		}
		
		ant.property( name : "global.artifact.location", value : ejbDest )	
		
	}
		
}